package org.jenkinsci.plugins.p4_client.changes;

import hudson.model.User;
import hudson.scm.ChangeLogSet;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.jenkinsci.plugins.p4_client.client.ClientHelper;

import com.perforce.p4java.core.ChangelistStatus;
import com.perforce.p4java.core.IJob;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.impl.generic.core.Changelist;

public class P4ChangeEntry extends ChangeLogSet.Entry {

	private Changelist changelist;
	private boolean shelved;
	private List<IFileSpec> files;
	private List<IJob> jobs;

	public P4ChangeEntry(P4ChangeSet parent) {
		super();
		setParent(parent);
	}

	@Override
	public String getMsg() {
		return changelist.getDescription();
	}

	@Override
	public User getAuthor() {
		String id = changelist.getUsername();
		User user = User.get(id);
		return user;
	}

	@Override
	public Collection<String> getAffectedPaths() {
		List<String> files = new ArrayList<String>();
		try {
			List<IFileSpec> list = changelist.getFiles(true);
			for (IFileSpec item : list) {
				files.add(item.getDepotPathString());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return files;
	}

	public void setChange(ClientHelper p4, int id) {
		try {
			this.changelist = (Changelist) p4.getChange(id);

			if (changelist.getStatus() == ChangelistStatus.PENDING) {
				this.files = p4.loadShelvedFiles(id);
				this.shelved = true;
			} else {
				this.files = changelist.getFiles(true);
				this.shelved = false;
			}

			this.jobs = changelist.getJobs();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public Changelist getChange() {
		return changelist;
	}

	public List<IFileSpec> getFiles() {
		return files;
	}

	public boolean isShelved() {
		return shelved;
	}

	public List<IJob> getJobs() {
		return jobs;
	}

	public String getJobStatus(IJob job) {
		Map<String, Object> map = job.getRawFields();
		String status = (String) map.get("Status");
		return status;
	}

	public String getJobSummary(IJob job) {
		String summary = job.getDescription();
		if (summary.length() > 80) {
			summary = summary.substring(0, 80) + "...";
		}
		return summary;
	}
}
